"use client";
import { Input, Flex, useColorModeValue, FormLabel } from "@chakra-ui/react";
import BrandButton from "@/components/buttons/brandButton";
import { useRouter, usePathname, useSearchParams } from "next/navigation";
import { useState } from "react";
import { DownloadIcon } from "@chakra-ui/icons";
import MultiSelect, { MultiSelectOption } from "../fields/MultiSelect";
import useExportHandler from "@/app/hooks/useExportHandler";

interface FiltersProps {
  onExport?: (params: Record<string, string>) => Promise<any>;
  users?: MultiSelectOption[];
  ibcs?: MultiSelectOption[];
  batches?: MultiSelectOption[];
  roles?: MultiSelectOption[];
  searchLabel?: string;
  showExport?: boolean;
  showSearch?: boolean;
  showDates?: boolean;
  showIbcs?: boolean;
  showUsers?: boolean;
  showBatches?: boolean;
  showRoles?: boolean;
  attempt?: boolean;
  defaultMax?: string;
}

const Filters = ({
  onExport,
  users,
  ibcs,
  batches,
  roles,
  searchLabel = "Name",
  showDates = true,
  showExport = true,
  showSearch = true,
  showIbcs = true,
  showUsers = true,
  showBatches = true,
  showRoles = false,
  attempt = false,
  defaultMax,
}: FiltersProps) => {
  const inputText = useColorModeValue("gray.700", "gray.100");
  const textColorPrimary = useColorModeValue("secondaryGray.900", "white");
  const router = useRouter();
  const pathname = usePathname();
  const searchParams = useSearchParams();
  const [selectedUsers, setSelectedUsers] = useState<MultiSelectOption[]>([]);
  const [selectedIbcs, setSelectedIbcs] = useState<MultiSelectOption[]>([]);
  const [selectedRoles, setSelectedRoles] = useState<MultiSelectOption[]>([]);
  const [selectedBatches, setSelectedBatches] = useState<MultiSelectOption[]>(
    []
  );
  const [selectedAttempt, setSelectedAttempt] = useState<MultiSelectOption[]>([]);
  const [filters, setFilters] = useState({
    fromDate: searchParams.get("from_date") || "",
    toDate: searchParams.get("to_date") || "",
    q: searchParams.get("q") || "",
  });

  // Export handler
  const { isExporting, handleExport } = useExportHandler({ onExport });

  const handleFromDateChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFilters({ ...filters, fromDate: e.target.value });
  };

  const handleToDateChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFilters({ ...filters, toDate: e.target.value });
  };

  const handleSearchChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFilters({ ...filters, q: e.target.value });
  };

  const handleUsersChange = (selected: MultiSelectOption[]) => {
    setSelectedUsers(selected.map((item) => item));
  };

  const handleIbcsChange = (selected: MultiSelectOption[]) => {
    setSelectedIbcs(selected.map((item) => item));
  };

  const handleRolesChange = (selected: MultiSelectOption[]) => {
    setSelectedRoles(selected.map((item) => item));
  };

  const handleBatchesChange = (selected: MultiSelectOption[]) => {
    setSelectedBatches(selected.map((item) => item));
  };

  const handleAttemptChange = (selected: MultiSelectOption[]) => {
    setSelectedAttempt(selected.map((item) => item));
  };

  // Apply filters to the URL
  const handleApplyFilters = () => {
    // Create an updated query object from the current URL search params
    const updatedQuery = new URLSearchParams(window.location.search);

    // Add or update filters if they exist, remove them if they are empty
    if (filters.fromDate) {
      updatedQuery.set("from_date", filters.fromDate);
    } else {
      updatedQuery.delete("from_date");
    }

    if (filters.toDate) {
      updatedQuery.set("to_date", filters.toDate);
    } else {
      updatedQuery.delete("to_date");
    }

    if (filters.q) {
      updatedQuery.set("q", filters.q);
    } else {
      updatedQuery.delete("q");
    }

    if (selectedUsers.length > 0) {
      // Extract `value` from selected users and set it in the query
      updatedQuery.set(
        "user_ids",
        selectedUsers.map((user) => user.value).join(",")
      );
    } else {
      updatedQuery.delete("user_ids");
    }

    if (selectedIbcs.length > 0) {
      // Extract `value` from selected IBCs and set it in the query
      updatedQuery.set(
        "ibc_ids",
        selectedIbcs.map((ibc) => ibc.value).join(",")
      );
    } else {
      updatedQuery.delete("ibc_ids");
    }

    if (selectedAttempt.length > 0) {
      // Extract `value` from selected IBCs and set it in the query
      updatedQuery.set(
        "attempt_ids",
        selectedAttempt.map((ibc) => ibc.value).join(",")
      );
    } else {
      updatedQuery.delete("attempt_ids");
    }

    if (selectedBatches.length > 0) {
      // Extract `value` from selected Batches and set it in the query
      updatedQuery.set(
        "batch_ids",
        selectedBatches.map((batch) => batch.value).join(",")
      );
    } else {
      updatedQuery.delete("batch_ids");
    }

    if (selectedRoles.length > 0) {
      // Extract `value` from selected Roles and set it in the query
      updatedQuery.set(
        "role_ids",
        selectedRoles.map((role) => role.value).join(",")
      );
    } else {
      updatedQuery.delete("role_ids");
    }

    // Build the new URL with the updated query parameters
    const newUrl = `?${updatedQuery.toString()}`;

    // Update the URL without refreshing the page
    router.push(newUrl);
  };

  const clearQueryParams = () => {
    // Clear the query parameters in the URL
    router.push(pathname);

    // Reset the filters state
    setFilters({ fromDate: "", toDate: "", q: "" });
    setSelectedUsers([]);
    setSelectedIbcs([]);
    setSelectedBatches([]);
    setSelectedRoles([]);
  };

  const today = new Date();
  const localToday = today.toLocaleDateString('en-CA'); // Format as "YYYY-MM-DD"
  const maxDate = defaultMax || localToday;

  return (
    <Flex gap={4} px={4} flexDirection={"column"}>
      <Flex direction="row" flexWrap="nowrap" gap={2} overflow={"auto"} className="hide-scrollbar">
        {/* Dates Filter */}
        {showDates && (
          <>
            <Flex direction="column">
              <FormLabel
                display="flex"
                fontSize="sm"
                color={textColorPrimary}
                fontWeight="bold"
                _hover={{ cursor: "pointer" }}
              >
                From
              </FormLabel>
              <Input
                type="date"
                value={filters.fromDate}
                onChange={handleFromDateChange}
                placeholder="From Date"
                color={inputText}
                max={maxDate}
              />
            </Flex>

            <Flex direction="column">
              <FormLabel
                display="flex"
                fontSize="sm"
                color={textColorPrimary}
                fontWeight="bold"
                _hover={{ cursor: "pointer" }}
              >
                To
              </FormLabel>
              <Input
                type="date"
                value={filters.toDate}
                onChange={handleToDateChange}
                placeholder="To Date"
                color={inputText}
                max={maxDate}
              />
            </Flex>
          </>
        )}

        {/* Search Filter */}
        {showSearch && (
          <Flex direction="column">
            <FormLabel
              display="flex"
              fontSize="sm"
              color={textColorPrimary}
              fontWeight="bold"
              _hover={{ cursor: "pointer" }}
            >
              {searchLabel}
            </FormLabel>
            <Input
              type="search"
              value={filters.q}
              onChange={handleSearchChange}
              placeholder="Search..."
              color={inputText}
              minW={"200px"}
              maxW={"300px"}
            />
          </Flex>
        )}

        {/* Users Filter */}
        {showUsers && (
          <Flex direction="column" minW={"200px"} maxW={"300px"}>
            <MultiSelect
              id="user-select"
              fieldName="user"
              label="Users"
              options={users}
              placeholder="Select User"
              defaultValue={selectedUsers}
              onChange={handleUsersChange}
            />
          </Flex>
        )}

        {/* IBCs Filter */}
        {showIbcs && (
          <Flex direction="column" minW={"200px"} maxW={"300px"}>
            <MultiSelect
              id="ibcs-select"
              fieldName="ibc"
              label="Ibcs"
              options={ibcs}
              placeholder="Select Ibcs"
              defaultValue={selectedIbcs}
              onChange={handleIbcsChange}
            />
          </Flex>
        )}

        {/* Roles Filter */}
        {showRoles && (
          <Flex direction="column" minW={"200px"} maxW={"300px"}>
            <MultiSelect
              id="roles-select"
              fieldName="roles"
              label="Roles"
              options={roles}
              placeholder="Select Roles"
              defaultValue={selectedRoles}
              onChange={handleRolesChange}
            />
          </Flex>
        )}

        {/* Batches Filter */}
        {showBatches && (
          <Flex direction="column" minW={"200px"} maxW={"300px"}>
            <MultiSelect
              id="batches-select"
              fieldName="batches"
              label="Batches"
              options={batches}
              placeholder="Select Batches"
              defaultValue={selectedBatches}
              onChange={handleBatchesChange}
            />
          </Flex>
        )}

        {attempt && (
          <Flex direction="column" minW={"200px"} maxW={"300px"}>
            <MultiSelect
              id="attempt-select"
              fieldName="attempt"
              label="Attempt"
              options={[{ label: "Attempt", value: "1" }, { label: "Not Attempt", value: "2" }]}
              placeholder="Select Please"
              defaultValue={selectedBatches}
              onChange={handleAttemptChange}
            />
          </Flex>
        )}


      </Flex>

      {/* Actions */}
      <Flex direction="row" gap={2} justifyContent={"flex-end"}>
        <BrandButton onClick={handleApplyFilters} buttonText="Apply" />
        <BrandButton onClick={clearQueryParams} buttonText="Clear" />
        {showExport && (
          <BrandButton
            icon={DownloadIcon}
            buttonText="Export"
            isLoading={isExporting}
            onClick={handleExport}
          />
        )}
      </Flex>
    </Flex>
  );
};

export default Filters;